#include <SPI.h>
#include "src/Adafruit_GFX/Adafruit_GFX.h"
#include "src/Adafruit_SSD1306/Adafruit_SSD1306.h"
#include "src/IQS269/IQS269.h"
#include "src/IQS269/IQS269A_Linearised.h"

/*** Defines ***/
//For serial debugging define DEBUG
//#define DEBUG

// AZP985A02 Requirements
#define IQS_RDY 2
#define IQS_ADR 0x44
#define BS0     A0
#define IO_1    A1
#define BS1     A2
#define BS2     A3
#define BTN0    3
#define BTN1    4

// OLED Requirements
#define OLED_GPIO   8
#define OLED_MOSI   9
#define OLED_CLK   10
#define OLED_DC    11
#define OLED_CS    12
#define OLED_RESET 13

// Coordinates
#define X_MIDDLE  63
#define Y_MIDDLE  31

#define BUTTON  30

/*** Types ***/

typedef union
{
  uint8_t modeByte;
  struct{
    bool touch:1;
    bool _slider:1;
    bool coil:1;
    bool ref:1;
    bool coilA:1;
    bool coilB:1;
    bool coilC:1;  
    bool reservedForPrevious:4;
  }current;
  struct{
    bool reservedForCurrent:4;
    bool touch:1;
    bool _slider:1;
    bool coil:1;
    bool coilA:1;
    bool coilB:1;
    bool coilC:1;
  }previous;
} Mode_t;

typedef union
{
  uint8_t powerByte;
  struct{
    bool np:1;
    bool lp:1;
    bool ulp:1;
    bool reserved;
  }mode;
} Power_t;

typedef union
{
  uint8_t bsByte;
  struct{
    bool _0:1;
    bool _1:1;
    bool _2:1;
    bool prev0:1;
    bool prev1:1;
    bool prev2:1;
    bool _8button:1;
    bool _2button:1;
    bool _inductive:1;
  }bs;
} BoardSelect_t;
    
/*** Macros ***/
//#define mode (*(Mode_t *) &GPIOR0)

Mode_t mode;
Power_t power;
BoardSelect_t board;

/*** Instances ***/
IQS269 iqs; // Creat an IQS269 instance

Adafruit_SSD1306 display(OLED_MOSI, OLED_CLK, OLED_DC, OLED_RESET, OLED_CS);

/*** Variables ***/
unsigned int firstMODETime = 0;
unsigned int buttonMODETimer = 0;
bool buttonMODEstate = false;

unsigned int firstBTN1Time = 0;
unsigned int buttonBTN1Timer = 0;
bool buttonBTN1state = false;

unsigned int firstGestureTime = 0;
unsigned int gestureTimer = 0;
bool gestureDisplaystate = false;

volatile bool skipStartup = false;
volatile bool startupCoilA = false;

short coil = 0;

void setup()
{
  // Start Serial communication.
  //#ifdef DEBUG
  //Serial.begin(115200);
  //Serial.println("Serial debugging");
  //#endif
  
  //Enable power to VBAT
  pinMode(OLED_GPIO, OUTPUT);
  digitalWrite(OLED_GPIO, HIGH);
  delay(500);

  // Start OLED.
  display.begin(SSD1306_SWITCHCAPVCC);
  //display.invertDisplay(true);
  display.display();
  delay(3000);
  display.clearDisplay();

  //Initialize BS0 (used for board selection)
  pinMode(BS0, INPUT);
  board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
  //Initialize BS1 (used for board selection)
  pinMode(BS1, INPUT);
  board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
  //Initialize BS2 (used for board selection)
  pinMode(BS2, INPUT);
  board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011

  //Initialize BTN0 (used for Mode select)
  pinMode(BTN0, INPUT);
  buttonMODEstate = digitalRead(BTN0);
  //Initialize BTN1 (used for Function select)
  pinMode(BTN1, INPUT);
  buttonBTN1state = digitalRead(BTN1);
  
  //Initialize IO_1 (used for HEAT)
  pinMode(IO_1, OUTPUT);
  digitalWrite(IO_1, LOW);   //HEAT => 

  //#ifdef DEBUG
  //Serial.println("IQS269A Setup start");
  //#endif
  
  // Do a startup routine.
  attachInterrupt(digitalPinToInterrupt(BTN0), endStartup, CHANGE);
  oledStartupRoutine_inductive();
  skipStartup = true;
  
  // Initialize the IQS sensor.
  while(!iqs.begin(IQS_ADR, IQS_RDY));

  //#ifdef DEBUG
  //Serial.println("IQS269A Ready");
  //#endif

  if(iqs.checkProductNum(RESTART) == false)
  {
    //#ifdef DEBUG
    //Serial.println("Device is not a IQS269A");
    //#endif

    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
    oledDisplayText("Device connected is    not an IQS269A", 5, 35, 1, WHITE);
    display.display();
    while(1);
  }

  if(iqs.checkSoftwareNum(RESTART) == false)
  {
    //#ifdef DEBUG
    //Serial.println("IQS269A firmware outdated");
    //#endif

    oledDisplayText("!", X_MIDDLE, 5, 2, WHITE);
    oledDisplayText("Outdated IQS269A FW", 10, 30, 1, WHITE);
    oledDisplayText("IQS269A v1 or later", 10, 50, 1, WHITE);
    display.display();
    while(1);
  }
/*------------board selct NA------------------*/
//  if( !(board.bs._0) && (board.bs._1) && (board.bs._2) )
//  {
//    //continue as normal
//    board.bs._8button = true;
//  }
//  else
//  {
//    board.bs._8button = false;
//    
//    display.clearDisplay();
//    display.display();
//    delay(100);
//
//    oledDisplayText("8 Button    board   required!", 10, 10, 2, WHITE);
//    display.display();
//    delay(1000);
//    setup();    
    loop();
//  }

  //Write MM
  iqs.writeMM(RESTART);

  //Enable the partial Hall UI
  iqs.enableHallUI(RESTART);

  // Acknowledge Reset.
  iqs.acknowledgeReset(RESTART);
  
  // Autone after making settings.
  iqs.autoTune(STOP);
  delay(500);
  
  // Display that initialization is complete and sensor is ready.
  while(!iqs.waitForReady());
  // Acknowledge Reset.
  iqs.acknowledgeReset(STOP);

  //#ifdef DEBUG
  //Serial.println("IQS269A Ready");
  //#endif
  
  display.clearDisplay();

  //Initialize zero states
    mode.current.touch = false;
    mode.current._slider = false;
    mode.current.coil = true;
    mode.current.coilA = false;
    mode.current.coilB = false;
    mode.current.coilC = false;
    
    mode.previous.touch = false;
    mode.previous._slider = false;
    mode.previous.coil = false;
    mode.previous.coilA = true;
    mode.previous.coilB = false;
    mode.previous.coilC = false;
    startupCoilA = false;

  //Always start in NP
    power.mode.np = true;
    power.mode.lp = false;
    power.mode.ulp = false;

    
    //#ifdef DEBUG
    //Serial.println("Normal Power mode");
    //#endif

    // Update OLED.
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText("xxuA", X_MIDDLE+2, 2, 1, WHITE);

    oledDisplayText("Hall", 0, 2, 1, WHITE);
//    oledDisplayText("Mode: Coil", 25, 54, 1, WHITE);
} //setup   

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void loop()
{
  signed int ch0CS = 0;
  signed int ch0LTA = 0;
  signed int ch0Delta = 0;  
  signed int ch1CS = 0;
  signed int ch1LTA = 0;
  signed int ch1Delta = 0;
  signed int ch2CS = 0;
  signed int ch2LTA = 0;
  signed int ch2Delta = 0;
  signed int ch7Delta = 0;

  if(!startupCoilA)
  {
    mode.current.coilA = true;
    startupCoilA = true;
  }
  
  //Buffer last board select
  board.bs.prev0 = board.bs._0;
  board.bs.prev1 = board.bs._1;
  board.bs.prev2 = board.bs._2;
  
  //Read BS0,BS1,BS2 (used for board selection)
  board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
  board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
  board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011
  
/*----------board select NA for inductive  
//  if( (!board.bs._0) && (board.bs._1) && (board.bs._2) )
//  {
//    //continue as normal
//    board.bs._inductive = true;
//  }
//  else
//  {
//    board.bs._8button = false;
//
//    do
//    {
//      display.clearDisplay();
//      display.display();
//      delay(100);
//  
//      oledDisplayText("8 Button    board   required!", 10, 10, 2, WHITE);
//      display.display();
//      delay(1000);
//      
//      //Buffer last board select
//      board.bs.prev0 = board.bs._0;
//      board.bs.prev1 = board.bs._1;
//      board.bs.prev2 = board.bs._2;
//      
//      //Read BS0,BS1,BS2 (used for board selection)
//      board.bs._0 = digitalRead(BS0);   //BS[2;1;0] => 110
//      board.bs._1 = digitalRead(BS1);   //BS[2;1;0] => 101
//      board.bs._2 = digitalRead(BS2);   //BS[2;1;0] => 011
//    }while(board.bs._0);
//        
//    setup();
//  }
---------------*/
  
  //Read BTN0 (used for Mode select)
  buttonMODEstate = digitalRead(BTN0);
  //Read BTN1 (used for Function select)
  buttonBTN1state = digitalRead(BTN1);
  
  // Read sensor information.
  while(!iqs.waitForReady());
  iqs.updateSystemFlags(RESTART);
  iqs.updateEvents(RESTART);
  iqs.updateProxByte(RESTART);
  iqs.updateTouchByte(RESTART);
  iqs.updateDeepByte(RESTART);

  //if slider mode is active, setup coordinates
  if(mode.current._slider)
  {
    iqs.updateGestures(RESTART);
    iqs.slider0_coordinate = iqs.readSlider0(RESTART);
    iqs.slider1_coordinate = 255 - iqs.readSlider1(RESTART); //CH0 & CH1 swopped
  }
  //if coil mode is active, setup .....
  else if(mode.current.coil)
  {

  }
  
  //For now: Coil A = CH0; Coil B = CH1; Coil C = CH2; Hall = CH7
  ch0CS = iqs.readChannelCounts(CH_0, RESTART);
  ch0LTA = iqs.readChannelLta(CH_0, RESTART);
  ch0Delta = (signed)(ch0LTA - ch0CS);
  
  ch1CS = iqs.readChannelCounts(CH_1, RESTART);
  ch1LTA = iqs.readChannelLta(CH_1, RESTART);
  ch1Delta = (signed)(ch1LTA - ch1CS);
  
  ch2CS = iqs.readChannelCounts(CH_2, RESTART);
  ch2LTA = iqs.readChannelLta(CH_2, RESTART);
  ch2Delta = (signed)(ch2LTA - ch2CS);
    
  ch7Delta = (signed)(iqs.readChannelDelta(CH_7, STOP));
  oledBar(0, ch7Delta);
  
/*-------------start inductive demo--------------*/
/*------------------Slider Mode (NA)------------------*/
  if(mode.current._slider)
  {
    //display slider mode and coil B - might change to x coord
    oledDisplayText("Mode: Slider", 35, 54, 1, WHITE);
    oledDisplayText("Coil B", 90, 2, 1, WHITE);
    if(gestureDisplaystate)
    {
      gestureTimer = millis() - firstGestureTime;
    }

  } 
/*----------------end slider mode---------------*/
  
/*-------------------Coil Mode------------------*/
  else if(mode.current.coil)
  {
    //clear current mode name and display coil mode
    oledClearModeName();
    oledDisplayText("Mode: Coil", 35, 54, 1, WHITE);

/*--------------------coilA----------------------*/
    if(mode.current.coilA) 
    {
     //  oledDisplayText("Delta", 90, 25, 1, WHITE);
       oledDisplayText("Coil A", 90, 2, 1, WHITE);
       //#ifdef DEBUG
       //Serial.println(iqs.cap0);
       //#endif

      if(iqs.prox.ch0)
      {
        oledClearCoilDelta();
        oledDisplayText("Delta", 90, 25, 1, WHITE);
        oledDisplayText(String(ch0Delta), 90, 40, 1, WHITE);
        //coilA
        if(iqs.deep.ch0) //coilA - Deep
        {
          oledDisplayText("A", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);   
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 10, WHITE);
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 7, WHITE);
        }
        else if(iqs.touch.ch0) //coilA - Touch
        {
          oledClearCoilA();
          oledDisplayText("A", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);   
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 10, WHITE);
        }  
        else if(iqs.prox.ch0) //coilA - Prox
        {
          oledClearCoilA(); 
          oledDisplayText("A", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE); 
        }
        else //None
        {
          oledClearCoilA();
        }
      }
      //if no prox is detected, clear screen
      else
      {
        oledClearAllButtons();
        oledClearDeltaText();
      }
    }
/*------------------end coilA--------------------*/
/*--------------------coilB----------------------*/
    else if(mode.current.coilB) 
    {
      oledDisplayText("Coil B", 90, 2, 1, WHITE);

       //#ifdef DEBUG
       //Serial.println(iqs.cap0);
       //#endif

      if(iqs.prox.ch1)
      {
        oledClearCoilDelta();
        oledDisplayText("Delta", 90, 25, 1, WHITE);
        oledDisplayText(String(ch1Delta), 90, 40, 1, WHITE);
        //coilB
        if(iqs.deep.ch1) //coilB - Deep
        {
          oledDisplayText("B", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);   
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 10, WHITE);
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 13, WHITE);
        }
        else if(iqs.touch.ch1) //coilB - Touch
        {
          oledClearCoilB();
          oledDisplayText("B", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);   
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 13, WHITE);
        }  
        else if(iqs.prox.ch1) //coilB - Prox
        {
          oledClearCoilB(); 
          oledDisplayText("B", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE); 
        }
        else //None
        {
          oledClearCoilB();
          oledClearDeltaText();
        }
      }
      //if no prox is detected, clear screen
      else
      {
        oledClearAllButtons();
        oledClearDeltaText();
      }
    }
/*------------------end coilB--------------------*/
/*--------------------coilC----------------------*/
    else if(mode.current.coilC) 
    {
      oledDisplayText("Coil C", 90, 2, 1, WHITE);

       //#ifdef DEBUG
       //Serial.println(iqs.cap0);
       //#endif

      if(iqs.prox.ch2)
      {
        oledClearCoilDelta();
        oledDisplayText("Delta", 90, 25, 1, WHITE);
        oledDisplayText(String(ch2Delta), 90, 40, 1, WHITE);
        //coilB
        if(iqs.deep.ch2) //coilC - Deep
        {
          oledDisplayText("C", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);   
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 13, WHITE);
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 16, WHITE);
        }
        else if(iqs.touch.ch2) //coilC - Touch
        {
          oledClearCoilC();
          oledDisplayText("C", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);   
          display.drawCircle(X_MIDDLE, Y_MIDDLE, 16, WHITE);
        }  
        else if(iqs.prox.ch2) //coilC - Prox
        {
          oledClearCoilC(); 
          oledDisplayText("C", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE); 
        }
        else //None
        {
          oledClearCoilC();
          oledClearDeltaText();
        }
      }
      //if no prox is detected, clear screen
      else
      {
        oledClearAllButtons();
        oledClearDeltaText();
      }
    }
/*------------------end coilC--------------------*/
  }
/*-----------------end coil mode-----------------*/
/*--------------end inductive demo---------------*/

/*---------board select N/A-----------*/
/*  if(!board.bs._8button)
  {
    display.clearDisplay();
    display.display();
    delay(100);

    oledDisplayText("Inductive Button    board   required!", 10, 10, 2, WHITE);
    display.display();
    delay(1000);
        
    setup();
  }*/

/*--------------Mode Button Check----------------*/
    if(!buttonMODEstate) //MODE => ABS
    { 
      digitalWrite(IO_1, LOW);    
      if(!buttonMODEstate && buttonMODETimer == 0)
      { 
        // Reset BTN0 timer.
        firstMODETime = millis();
        delay(1);
      }
  
      buttonMODETimer = millis() - firstMODETime;
      
      if (buttonMODETimer > 500 && !buttonMODEstate)
      {
        if(!mode.current.coil)
        {
          //Activate the Absolute Cap mode
          mode.current._slider = false;
          mode.current.coil = true;
          mode.current.coilA = true;
  
          //#ifdef DEBUG
          //Serial.println("ABSOLUTE CAP activated");
          //#endif
          
          // Display ModeName on OLED.
          oledDisplayText("Mode: Coil", 35, 54, 1, WHITE);
          oledClearFunc();
          oledDisplayText("Coil A", 90, 2, 1, WHITE);
          coil = 0;
          mode.current.coilA = true;

          //Set to 4MHz FOSC; Xfer = 125kHz
          while(!iqs.waitForReady());
          //iqs.setFOSC(true, RESTART);
          // Autone after making settings.
          iqs.autoTune(STOP);

          //Enable the UI
          while(!iqs.waitForReady());
          buttonMODETimer = 0;
        }
  
        else if(mode.current.coil)
        {
/*
          //De-activate the Absolute Cap mode
          mode.current._slider = true;
          mode.current.coil = false;
  
          //#ifdef DEBUG
          //Serial.println("ABSOLUTE CAP de-activated");
          //#endif

          //Disable the UI
          while(!iqs.waitForReady());
    
          //Set back to 16MHz FOSC
          iqs.setFOSC(false, RESTART);
          // Autone after making settings.
          iqs.autoTune(STOP);
          while(!iqs.waitForReady());

          // Display ModeName on OLED.
          oledClearModeName();
          oledClearFunc();        
          oledClearCoilDelta();
          oledClearAllButtons();
          oledDisplayText("Mode: Slider", 35, 54, 1, WHITE);
*/        
          buttonMODETimer = 0;
        }
      }
    }
    else if(buttonMODEstate)
    {
      digitalWrite(IO_1, HIGH);
      buttonMODETimer = 0;
    }
/*------------end Mode Button Check--------------*/

/*--------------BTN1 Button Check----------------*/

    if(!buttonBTN1state && mode.current.coil) //BTN1 => Cycle ABS channel (CH0-> CH5 -> CH0)
    {      
      if(!buttonBTN1state && buttonBTN1Timer == 0)
      { 
        // Reset BTN1 timer.
        firstBTN1Time = millis();
        delay(1);
      }
  
      buttonBTN1Timer = millis() - firstBTN1Time;
      
      if (buttonBTN1Timer > 500 && !buttonBTN1state)
      {
        oledClearFunc();        
        coil++;
        
        if(coil == 3)
        {
          coil = 0;
        }
               
        while(!iqs.waitForReady());
        switch(coil)
        {
          case 0:
          oledClearFunc();
          oledClearAllButtons();
          oledDisplayText("Coil A", 90, 2, 1, WHITE);
          mode.current.coilA = true;
          mode.current.coilB = false;
          mode.current.coilC = false;
          
          break;

          case 1:
          oledClearFunc();
          oledClearAllButtons();
          oledDisplayText("Coil B", 90, 2, 1, WHITE);
          mode.current.coilA = false;
          mode.current.coilB = true;
          mode.current.coilC = false;
          break;

          case 2:
          oledClearFunc();
          oledClearAllButtons();
          oledDisplayText("Coil C", 90, 2, 1, WHITE);
          mode.current.coilA = false;
          mode.current.coilB = false;
          mode.current.coilC = true\
          ;
          break;

          default:
          oledClearFunc();
          break;
        }
       
        buttonBTN1Timer = 0;
      }
    }
    else if(buttonBTN1state)
    {
      buttonBTN1Timer = 0;
    }
/*------------end BTN1 Button Check--------------*/

/*-------------Power mode switching--------------*/
  //LP
  if (iqs.eventFlags.power && iqs.systemFlags.lp && !iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = false;
    power.mode.lp = true;
    power.mode.ulp = false;
    
    //#ifdef DEBUG
    //Serial.println("LP Mode");
    //#endif
    
    // Update display.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("LP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText("xxuA", X_MIDDLE+2, 2, 1, WHITE);
  }

  //ULP
  else if (iqs.eventFlags.power && iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = false;
    power.mode.lp = false;
    power.mode.ulp = true;
    
    //#ifdef DEBUG
    //Serial.println("ULP Mode");
    //#endif
    
    // Update display.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 20, 10, 2, WHITE);
    oledDisplayText("ULP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText(" xuA", X_MIDDLE+2, 2, 1, WHITE);
  }

  //NP
  if (!iqs.systemFlags.lp && !iqs.systemFlags.ulp)
  {
    // Update mode flags.
    power.mode.np = true;
    power.mode.lp = false;
    power.mode.ulp = false;
    
    //#ifdef DEBUG
    //Serial.println("NP mode");
    //#endif
    
    // Update OLED.
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE-13, 2, 1, BLACK);
    //oledDisplayText("xxuA", X_MIDDLE+2, 2, 1, WHITE); 
  }
/*-----------end Power mode switching------------*/

  //Single display update at the end
  display.display();
}
/*----------------end Main loop------------------*/

/* @brief Displays the Delta parameter on the OLED display at x position. */
void oledBar(uint8_t x, int16_t delta)
{
  //clear bar area
  display.fillRect(x, 10, 19, 54, BLACK);
  //center line
  display.drawLine(x, Y_MIDDLE, x+19, Y_MIDDLE, WHITE);

  //clear delta
  display.fillRect(x, 54, 23, 64, BLACK);
  oledDisplayText(String(abs(delta)), x, 54, 1, WHITE);

  if(delta/3 > 21) //Just to control the max
  {
     delta = 21*3;
  }
  else if(delta/3 < -21)
  {
     delta = -21*3;
  }
  
  if(delta > 0)
  {
    display.fillRect(x+6, Y_MIDDLE-(delta/3), 8, (delta/3), WHITE);
    if(delta > 10)
    oledDisplayText("+", x, Y_MIDDLE-10, 1, WHITE);
  }
  else
  {
    display.fillRect(x+6, Y_MIDDLE, 8, (abs(delta)/3), WHITE);
    if(delta < -10)
    oledDisplayText("-", x, Y_MIDDLE+5, 1, WHITE);
  }
}

/* @brief Clears the OLED display. */
void oledClear()
{
  display.clearDisplay();
}

/* @brief Clears the OLED coilA display. */
void oledClearCoilA()
{
    display.fillCircle(X_MIDDLE, Y_MIDDLE, 12, BLACK);
}

/* @brief Clears the OLED coilB display. */
void oledClearCoilB()
{
    display.fillCircle(X_MIDDLE, Y_MIDDLE, 15, BLACK);
}

/* @brief Clears the OLED coilC display. */
void oledClearCoilC()
{
    display.fillCircle(X_MIDDLE, Y_MIDDLE, 18, BLACK);
}

/* @brief Clears the OLED Button area display. */
void oledClearAllButtons()
{
    display.fillCircle(X_MIDDLE, Y_MIDDLE, 18, BLACK);
}

/* @brief Clears the OLED Mode name display. */
void oledClearModeName()
{
  display.fillRect(25, 54, 103, 10, BLACK);
}

/* @brief Clears the OLED Func display. */
void oledClearFunc()
{
  display.fillRect(90, 2, 40, 10, BLACK);
}

/* @brief Clears the OLED PM display. */
void oledClearPM()
{
  display.fillRect(X_MIDDLE-15, 0, 40, 12, BLACK);
}

/* @brief Clears the OLED Gesture display. */
void oledClearGesture()
{
  display.fillRect(101, 30, 27, 20, BLACK);
}

/* @brief Clears the OLED Coordinate display. */
void oledClearCoordinate()
{
  display.fillRect(40, 56, 88, 20, BLACK);
}

void oledClearCoilDelta()
{
  display.fillRect(90, 40, 30, 10, BLACK);
}

/* @brief Clears "Delta" from display */

void oledClearDeltaText()
{
  display.fillRect(90, 25, 40, 25, BLACK);
}


/*--------------Startup routine----------------*/
void oledStartupRoutine_inductive(void)
{
  display.clearDisplay();
  display.display();

  //PM demonstrations
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("NP", X_MIDDLE-13, 2, 1, BLACK);
    display.display();
    delay(500);
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 15, 10, 2, WHITE);
    oledDisplayText("LP", X_MIDDLE-13, 2, 1, BLACK);
    display.display();
    delay(500);
    oledClearPM();
    display.fillRoundRect(X_MIDDLE-15, 1, 20, 10, 2, WHITE);
    oledDisplayText("ULP", X_MIDDLE-13, 2, 1, BLACK);
    display.display();
    delay(500);

    //exit on button press
    //if button pressed return to main loop
    if (skipStartup) 
    {
      return;
    }

    // Display MODE on OLED.
    oledDisplayText("Mode: Coil", 35, 54, 1, WHITE);
    oledDisplayText("Coil x", 90, 2, 1, WHITE);  
    display.display();
    delay(500);
    oledClearFunc();
    display.display();
    delay(500);  

    //exit on button press
    //if button pressed return to main loop
    if (skipStartup) 
    {
      return;
    }
    
    //Coil A
    oledDisplayText("Coil A", 90, 2, 1, WHITE);
    //display text for prox
    oledDisplayText("A", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);
    display.display();
    delay(500);  
    
    oledDisplayText("Delta", 90, 25, 1, WHITE);
    oledDisplayText("x", 90, 40, 1, WHITE);
    display.display();
    delay(700);   
    //display text and circle for touch
    display.drawCircle(X_MIDDLE, Y_MIDDLE, 10, WHITE);
    display.display();
    delay(500);
    //display text and double circle for deep touch       
    display.drawCircle(X_MIDDLE, Y_MIDDLE, 7, WHITE);
    display.display();
    delay(500); 
    //clear coil
    oledClearDeltaText();
    oledClearAllButtons(); 
    oledClearFunc();
    display.display();
    delay(500); 
    
    //exit on button press
    //if button pressed return to main loop
    if (skipStartup) 
    {
      return;
    }
    
    //Coil B
    oledDisplayText("Coil B", 90, 2, 1, WHITE);
    //display text for prox
    oledDisplayText("B", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);
    display.display();
    delay(500);  
    
    oledDisplayText("Delta", 90, 25, 1, WHITE);
    oledDisplayText("x", 90, 40, 1, WHITE);
    display.display();
    delay(500);      
    //display text and circle for touch 

    display.drawCircle(X_MIDDLE, Y_MIDDLE, 13, WHITE);
    display.display();
    delay(500);
    //display text and double circle for deep touch       
    display.drawCircle(X_MIDDLE, Y_MIDDLE, 10, WHITE);
    display.display();
    delay(500); 
    //clear coil
    oledClearDeltaText(); 
    oledClearAllButtons();  
    oledClearFunc();
    display.display();
    delay(500);

    //exit on button press
    //if button pressed return to main loop
    if (skipStartup) 
    {
      return;
    }

    //Coil C
    oledDisplayText("Coil C", 90, 2, 1, WHITE);
    //display text for prox
    oledDisplayText("C", X_MIDDLE-2, Y_MIDDLE-3, 1, WHITE);
    display.display();
    delay(500);  
    
    oledDisplayText("Delta", 90, 25, 1, WHITE);
    oledDisplayText("x", 90, 40, 1, WHITE);
    display.display();
    delay(500);
    //display text and circle for touch      
    display.drawCircle(X_MIDDLE, Y_MIDDLE, 16, WHITE);
    display.display();
    delay(500);
    //display text and double circle for deep touch       
    display.drawCircle(X_MIDDLE, Y_MIDDLE, 13, WHITE);
    display.display();
    delay(500);    
    //clear coil
    oledClearDeltaText(); 
    oledClearAllButtons(); 
    oledClearFunc();
    display.display();
    delay(500);

    //exit on button press
    //if button pressed return to main loop
    if (skipStartup) 
    {
      return;
    }

    // Hall demonstartion.
    oledDisplayText("Hall", 0, 2, 1, WHITE);
    oledDisplayText("+", 0, Y_MIDDLE-10, 1, WHITE);
    
    //center line
    display.drawLine(0, Y_MIDDLE, 19, Y_MIDDLE, WHITE);
    display.display();
    delay(100);
    for(int count = 0; count < 20; count++)
    {
      display.fillRect(0, 10, 19, 20, BLACK);
      display.fillRect(6, Y_MIDDLE-(count), 8, (count), WHITE);
      display.display();
      delay(10);
    }

    //exit on button press
    //if button pressed return to main loop
    if (skipStartup) 
    {
      return;
    }
    //clear bar area
    display.fillRect(0, 10, 19, 21, BLACK);
    oledDisplayText("-", 0, Y_MIDDLE+5, 1, WHITE);
    display.display();
    delay(100);
    
    for(int count = 0; count > -20; count--)
    {
      display.fillRect(6, Y_MIDDLE, 8, abs(count), WHITE);
      display.display();
      delay(10);
    }
    delay(300);

  oledClearModeName();   
  display.clearDisplay();
  display.display();
//  delay(100);
}

void oledDisplayText(String text, unsigned char xCoord, unsigned char yCoord, unsigned char textSize, unsigned char color)
{
  display.setTextSize(textSize);
  display.setTextColor(color);
  display.setCursor(xCoord, yCoord);
  display.println(text);
}

void endStartup() 
{
  detachInterrupt(digitalPinToInterrupt(BTN0));
  skipStartup = true;
}
