# IQS269 Library
The IQS269 library is a library which allows easy setup and interaction with the Azoteq IQS266, 8 Channel self/projected
capacitive controller. The IQS269 is ideal for small buttons 7 sliders in wearables as it provides easy on chip
gesture detection as well as slider coordinate calculation. 

The IQS269 requires and I2C master to setup for custom application requirements.
See the list of functions below.

Since the IQS269 interacts with the MCU via I2C communication, this library is based on the standard Arduino
"Wire.h" library and is thus supported by any hardware which is supported by the "Wire.h" library. The "Wire.h" library
is included within the "IQS269.h" library and does not need to be included again. Furthermore, a Wire instance is 
created in the IQS269::begin() function and does not need to be created in the Setup() section of the Arduino sketch. 
The user does not need to interact with the "Wire.h" library at all.

# IQS269 Device
Information on the IQS269 device can be found her: https://www.azoteq.com/product/iqs269/
The datasheet can be downloaded from this link: https://www.azoteq.com/product/iqs269/
Any application notes can be found here: https://www.azoteq.com/design/application-notes/

# Methods:
Method names are mostly self explanatory. However, there is also a short commented header above each method
in the "IQS269.cpp" file which offers a brief explanation and description of parameters and return values. Also
see the "IQS269.h" file for all the method prototypes.

begin
requestComms
waitForReady
checkReset
checkProductNum
checkSoftwareNum
acknowledgeReset
autoTune
setStreamMode
setEventMode
readSystemFlags
updateSystemFlags
readEvents
updateEvents
readGestures
updateGestures
readTrackpadFlags
updateTrackPadFlags
readProxByte
readTouchByte
readChannelStatus
updateChannelStatus
readXCoordinate
readYCoordinate
setTouchSensitivity
setTouchThresholds
setProxSensitivity
setProxThreshold
enableProxEvent
enableTouchEvent
enableTrackPadEvent
enableTapEvent
enableSwipeEvent
enableAllEvents
disableProxEvent
disableTouchEvent
disableTrackPadEvent
disableTapEvent
disableSwipeEvent
disableAllEvents
enableChannel
disableChannel
readChannelCounts
readChannelLta
readChannelDelta
setTapTimerLimit
setTapThreshold
setSwipeTimerLimit
setSwipeThreshold
enableProxDistribute
disableProxDistribute
setEventMask
setProxAtiTarget
setTouchAtiTarget
setProxBaseValue
setTouchBaseValue
setReportRateNM
setReportRateLP
setNPSegmentRate
setZoomTimeout
disableHaltTimeout
setHaltTimeout

# I2C Communication:
The IQS269 communicates with the master MCU through I2C communication. However, the IQS269 has an additional "Ready" line which
can let the master know when it has information available. This is particularly useful as it allows the MCU to service the IQS269
on an interrupt basis rather than polling it. More on "Event Mode" and "Stream Mode" later.
NOTE: DO NOT PERFORM I2C COMMUNICATION IN AN INTERRUPT SERVICE ROUTINE, THIS CAUSES THE MCU TO CRASH. I don't know why.

Most functions of th IQS269 library have a boolean parameter as the last parameter named stopOrRestart. This boolean indicates
wether the current communication window must be kept open or closed. Use STOP or RESTART as defined in the library header.
When calling multiple functions in a block of code with the same IQS269 instance all functions except for the last should 
use RESTART and the last function should then use STOP. See the accompanied examples.

# Default Settings:
The default startup settings of th IQS269 work well. However, some settings such as touch sensitivity depend largely on the
trackpad design and the overlay on the touch interface. These settings can be adjusted at initilialzation after the begin
method has been called.

# Event Mode and Stream Mode:
The IQS269 starts up in Stream Mode, however, Event Mode is very convenient as the user can select which events are important.
Event mode means that the device only requests communication when it has detected an event as allowed by the enabled events.
Communication is requested by pulling the ready pin LOW, the ready pin will continue to toggle on and off until communication is established.
Event mode is very convenient as it allows the MCU to carry on with its own business until the IQS269 detects an event and pulls the ready pin LOW.
The ready pin on the Arduino can then trigger an external interrupt or a pin change interrupt.
NOTE: DO NOT PERFORM I2C COMMUNICATION IN AN INTERRUPT SERVICE ROUTINE, THIS CAUSES THE MCU TO CRASH. I don't know why.

In Stream Mode the IQS device pulls the ready line low in specified intervals which lets the MCU know that the latest sensing
information is now avialable to be read. This is done regardless of whether an event is detected or not. In this way the MCU can
decide when it wants to read information polling the ready line and reading the information it needs as soon as it is avilable. Unless
the MCU checks on the IQS device regularly it can miss events.

# Requesting Communication:
In stream mode the ready pin is pulled low frequently, when the ready line is LOW the MCU can innitiate communication with the IQS device.
The waitForReady() function can be used to wait for the ready pin to go LOW, the waitForReady() function waits until the ready pin is
LOW and then returns a TRUE. If the ready pin does not go LOW within 100ms it returns FALSE. The function can be put in a while loop to
continue calling it until the ready pin is pulled LOW.

In event mode there might be long periods where no event occurs, then the ready pin is not pulled low by the IQS device. The MCU can request
communication by briefly pulling the ready pin low and then waiting for the IQS device to pull the ready pin low. This is done by the
requestComms() function. A TRUE is returned if the IQS responds within 100ms, otherwise a FALSE is returned. Use the function in a while loop
to call it until communication is established. After the requestComms() function returns a TRUE, the MCU can proceed with other functions.
In event mode any block of IQS269 library functions should be preceeded by the requestComms() function unless it is certain that the IQS device
has solicited the communication. The last function in any block of functions should then use STOP as its final argument.

# Events
In Event Mode an event causes the IQS269 to pull the Ready pin LOW if that event is enabled. This event is then stored in the Events register
until the current communication window is closed.
In Stream Mode any occuring event is kept in the Events register until the next conversion cycle which then loads the new events.
The events Register can be read with the readEvents() function. This returns a byte. The definitions PROX_EVENT_BIT, TOUCH_EVENT_BIT, TP_EVENT_BIT,
ATI_EVENT_BIT, TAP_EVENT_BIT and SWIPE_EVENT_BIT, as defined in "IQS269.h", can be used to determine which event bits are set and thus
which events have ocurred.

To really understand how the events work the IQS269 datasheet can be consulted.
				
That covers the basics. When in doubt, consult the device datasheet. Azoteq also offers an evaluation kit, configuration tool and free GUI based PC software
which makes it easy to configure settings from basic to advanced and visualize what is happening inside the device.
See this page for the EV kit and PC software: hhttp://www.azoteq.com/design/evaluation-kits.html
